(* :Title: RettaFasci.m *)

(* :Context: RettaFasci` *)

(* :Author: Lorenzo Roi *)

(* :Summary: Definisce le funzioni di base che coinvolgono punti del piano cartesiano 
   rette e fasci di rette.
*)

(* :Copyright:  2004 *)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 3.0 *)

(* :Keywords: retta, fasci di rette *)

BeginPackage["RettaFasci`", "Utilities`FilterOptions`", "Graphics`ImplicitPlot`"];

testLinearita::usage = "Esegue i test necessari per riconoscere se equazione in
testLinearita[equazione, {varx,vary}] pu essere l'equazione rappresentativa
di una retta nelle variabili varx e vary. Nella forma testLinearita[equazione]
si presuppone l'uso delle variabili canoniche x e y e ritorna i valori True o False.";

riduceParametrica::usage = "riduceParametrica[rettaParam, param] elimina il parametro param
dalla lista rettaParam contenente le due equazioni della rappresentazione parametrica
e fornisce la corrispondente equazione implicita."

equazioneImplicita::usage = "La funzione equazioneImplicita[retta, {varx,vary}] ritorna
l'equazione di retta nelle variabili {varx, vary} scritta nella forma implicita.
Retta dev'essere un'equazione lineare.\nNella forma pi sintetica equazioneImplicita[retta],
si sottintende che le variabili siano x e y.";

equazioneEsplicita::usage = "La funzione equazioneEsplicita[retta, {varx,vary}] fornisce
l'equazione della retta nella forma esplicita nelle variabili varx e vary.
Retta dev'essere un'equazione lineare.\nNella forma pi sintetica equazioneEsplicita[retta]
si sottintende che le variabili siano x e y.";

coefficientiImplicita::usage = "Nella forma coefficientiImplicita[retta, {varx,vary}]
fornisce la lista dei coefficienti della forma implicita di retta nelle variabili {varx,vary}.
Retta dev'essere un'equazione lineare.
\nNella forma pi sintetica coefficientiImplicita[retta] le variabili sono quelle canoniche, x e y.";

coefficientiEsplicita::usage = "La funzione coefficientiEsplicita[retta, {varx,vary}]
restituisce la lista del coefficiente angolare e del termine noto della forma
esplicita di retta nelle variabili {varx,vary}. Retta dev'essere un'equazione lineare.
\nNella forma pi sintetica coefficientiEsplicita[retta] si sottintende che le
variabili siano x e y.";

coefficienteAngolare::usage = "coefficienteAngolare[retta, {varx,vary}] fornisce il coefficiente
angolare di retta nelle variabili {varx,vary}. Nella forma coefficienteAngolare[{x1,y1}, {x2,y2}]
calcola il coefficiente angolare della retta passante per i punti {x1,y1}, {x2,y2}.
\nNella forma pi sintetica coefficienteAngolare[retta] si sottintende che le variabili
siano x e y.";

graficoAnimazioneFascio::usage = "graficoAnimazioneFascio[fascio, intervallox, intervalloy, intervalloParam, opzioni]
genera l'animazione di fascio, al variare del parametro nell'insieme
di valori definito da intervalloParam cio {param, paramMin, paramMax}. L'intervallo dei valori di param pu essere anche
{param, paramMin, paramMax, paramIncremento}. Il grafico 
compreso negli intervalli
\nintervallox = {x, xmin, xmax} e intervalloy= {y, ymin, ymax}.
\n graficoAnimazioneFascio[retta, intervallox, intervalloy, intervalloParam, {x0,y0}, opzioni]
traccia pure il punto {x0,y0}.";

centroFascio::usage = "centroFascio[retta, param, {varx,vary}] fornisce il centro del fascio
proprio relativo al parametro param. Le variabili indipendente e dipendente sono {varx,vary}.
\nNella forma pi sintetica centroFascio[retta, param] si sottintende che le variabili
siano x e y.";

graficoRetteParametriche::usage = "graficoRetteParametriche[ListaRette, param, intervallox, intervalloy, opzioni]
traccia il grafico delle sole rette espresse in forma parametrica e comprese in
ListaRette e di parametro param. La finestra del piano cartesiano  definita da
\nintervallox = {x, xmin, xmax} e intervalloy = {y, ymin, ymax}.";

graficoRette::usage = "Nella forma graficoRette[ListaRette, param, intervallox, intervalloy, opzioni]
traccia il grafico di tutte le rette in forma esplicita, implicita o parametrica
(di parametro param) presenti in ListaRette.
\nNella forma graficoRette[ListaRette, intervallox, intervalloy, opzioni] d solo quelle
nella rappresentazione esplicita o implicita.";

generatriciFascio::usage = "generatriciFascio[fascio, param] restituisce la lista delle rette generatrici
del fascio espresso da fascio e di parametro param.";

graficoFascioGeneratrici::usage = "graficoFascioGeneratrici[fascio, intervallox, intervalloy, intervalloParam]
permette l'animazione del fascio rappresentato dall'equazione fascio, riportando pure
le rette generatrici del fascio e il relativo centro.
\n Il formato per intervallox  {x, xmin, xmax}, intervalloy = {y, ymin, ymax} e
intervalloParam = {param, paramMin, paramMax}.";

distanza2Punti::usage = "distanza2Punti[{x1,y1}, {x2,y2}] d la distanza tra due punti {x1,y1} e {x2,y2}.";

puntoMedio::usage = "puntoMedio[{x1, y1}, {x2, y2}] calcola le coordinate del punto medio del segmento
di estremi {x1, y1} e {x2, y2}.";

puntiCollineari::usage = "Impone la condizione di collinearit dei tre punti {x1, y1}, {x2,y2},
{x3, y3}. Se i tre punti sono collineari l'esito  True altrimenti False. Se sono presenti
dei valori simbolici restituisce l'equazione che dev'essere soddisfatta per la collinearit.";

retta2PuntiParametrica::usage = "retta2PuntiParametrica[{x1,y1}, {x2,y2}, param, {varx,vary}] costruisce
la retta nella rappresentazione parametrica relativamente al parametro param e
alla coppia di variabili varx e vary. Restituisce quindi una lista contenente le 
due equazioni parametriche.
\nNella forma retta2PuntiParametrica[{x1,y1}, {x2,y2}, param] vengono assunte le variabili
canoniche x e y.";

retta2Punti::usage = "retta2Punti[{x1,y1}, {x2, y2}] restituisce l'equazione implicita della
retta per i punti {x1, y1}, {x2, y2} nelle variabili x e y. La forma retta2Punti[{x1,y1},
{x2, y2}, {varx,vary}] permette invece di disporre di un'equazione nelle variabili arbitrarie varx e vary.";

rettaParallela::usage = "rettaParallela[{x0, y0}, retta, {varx,vary}] fornisce la forma implicita
della retta per il punto {x0,y0} e parallela a retta.
\nNella forma rettaParallela[{x0, y0}, retta] sono assunte implicitamente le variabili x e y.
\nNelle forme rettaParallela[{x0, y0}, coefficienteAngolare,  {varx,vary}],
rettaParallela[{x0, y0}, coefficienteAngolare] si sostituisce a retta il relativo
coefficiente angolare.";

rettaPerpendicolare::usage = "rettaPerpendicolare[{x0, y0}, retta, {varx,vary}] calcola la forma
implicita della retta per il punto {x0,y0} e perpendicolare a retta.
\nNella forma rettaPerpendicolare[{x0, y0}, retta] sono assunte implicitamente le variabili x e y.
\nNelle forme rettaPerpendicolare[{x0, y0}, coefficienteAngolare,  {varx,vary}],
rettaPerpendicolare[{x0, y0}, coefficienteAngolare] si sostituisce a retta il relativo
coefficiente angolare.";

distanzaPuntoRetta::usage = "distanza[{x0, y0}, retta, {varx,vary}] calcola la distanza
del punto {x0, y0} da retta. Quest'ultima  un'equazione nella variabili {varx,vary}.
Nella forma distanza[{x0, y0}, retta] sono assunte implicitamente le variabili x e y.";

puntoIntersezione::usage = "puntoIntersezione[retta1, retta2, {varx,vary}] calcola il punto di
intersezione delle rette retta1 e retta2 espresse nelle variabili varx e vary.
\npuntoIntersezione[retta1, retta2] considera le rette dipendenti dalle variabili
canoniche x e y. Le due rette si possono fornire anche comprese in una lista cio
come puntoIntersezione[{retta1, retta2}].";

appartenenzaPuntoRetta::usage = "appartenenzaPuntoRetta[{x0,y0}, retta, {varx,vary}] sostituisce
nell'equazione rappresentativa di retta alle variabili varx e vary le coordinate
del punto {x0,y0}.
\nNella forma appartenenzaPuntoRetta[{x0,y0}, retta] le variabili presenti in retta
sono quelle canoniche cio x e y.";

areaTriangolo::usage = "areaTriangolo[{x1, y1}, {x2, y2}, {x3, y3}] calcola l'area del triangolo
di vertici {x1, y1}, {x2, y2}, {x3, y3}.";

coloriDisponibili::usage = "Presenta una lista dei colori disponibili. Questi sono: arancione,
azzurro, blu, cyan, giallo, magenta, marrone, oro, rosa, rosso, verde, verdeSmeraldo, violetto."

colore::usage = "L'espressione colore[nomeColore] colora di nomeColore l'oggetto grafico associato."
    
(*
Sezione implementazione 
*)

Begin["`Private`"];

testLinearita[espressione_Equal, {x_Symbol,y_Symbol}] :=
   Module[{primoMembro, esitoTest}, 
      primoMembro = First[espressione]-Last[espressione];
      testLinearita::lin = "L'espressione `1` non \[EGrave] lineare nelle variabili `2` e `3`.";
      esitoTest = PolynomialQ[primoMembro, {x,y}]&&
                  (0 <= Exponent[primoMembro,x] <= 1)&&
                  (0 <= Exponent[primoMembro,y] <= 1)&&
                  (1 <= Exponent[primoMembro,x] + Exponent[primoMembro,y] <= 2);
      If[esitoTest, esitoTest, (Message[testLinearita::lin, espressione, x, y]; esitoTest)]
   ];
            
testLinearita[espressione_Equal] :=
   Module[{primoMembro, esitoTest},
      primoMembro = First[espressione]-Last[espressione];
      testLinearita::lin="L'espressione `1` non \[EGrave] lineare nelle variabili `2` e `3`.";
      esitoTest = PolynomialQ[primoMembro, {Global`x,Global`y}]&&
                  (0 <= Exponent[primoMembro, Global`x] <= 1)&&
                  (0 <= Exponent[primoMembro, Global`y] <= 1)&&
                  (1 <= Exponent[primoMembro, Global`x] + Exponent[primoMembro, Global`y] <= 2);
      If[esitoTest, esitoTest, (Message[testLinearita::lin, espressione, Global`x, Global`y]; esitoTest)]
   ];
          
riduceParametrica[rettaParam_List, param_Symbol] := Eliminate[rettaParam, param];
            
equazioneImplicita[retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{}, Collect[First[retta] - Last[retta], {x, y}] == 0];
      
equazioneImplicita[retta_ /; testLinearita[retta]] := 
   Module[{}, Collect[First[retta] - Last[retta], {Global`x, Global`y}] == 0];
      
equazioneEsplicita[retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{sol}, sol = Roots[retta, y]; sol /. Last[sol] -> Apart[Last[sol], x]];
      
equazioneEsplicita[retta_ /;testLinearita[retta]] := 
   Module[{sol}, sol = Roots[retta, Global`y]; sol /. Last[sol] -> Apart[Last[sol], Global`x]];
      
coefficientiImplicita[retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{primoMembro = First[equazioneImplicita[retta, {x, y}]]}, 
      {Coefficient[primoMembro, x], Coefficient[primoMembro, y], primoMembro /. {x -> 0, y -> 0}}
   ];
      
coefficientiImplicita[retta_ /;testLinearita[retta]] := 
   Module[{primoMembro = First[equazioneImplicita[retta, {Global`x, Global`y}]]}, 
      {Coefficient[primoMembro, Global`x], Coefficient[primoMembro, Global`y], 
      primoMembro /. {Global`x -> 0, Global`y -> 0}}
   ];
        
coefficientiEsplicita[retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   List[Coefficient[Last[equazioneEsplicita[retta, {x, y}]], x], 
   Last[equazioneEsplicita[retta, {x, y}]] /. x -> 0] 
    
coefficientiEsplicita[retta_ /;testLinearita[retta]] := 
   List[Coefficient[Last[equazioneEsplicita[retta, {Global`x, Global`y}]], Global`x], 
   Last[equazioneEsplicita[retta, {Global`x, Global`y}]] /. Global`x -> 0]
      
coefficienteAngolare[retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Simplify[First[coefficientiEsplicita[retta, {x, y}]]] 
    
coefficienteAngolare[retta_ /; testLinearita[retta]] := 
   Simplify[First[coefficientiEsplicita[retta, {Global`x, Global`y}]]]
  
coefficienteAngolare[{x1_, y1_}, {x2_, y2_}] := (y2 - y1)/(x2 - x1);          
          
(* 
Nelle successive funzioni grafiche, non vengono fatti controlli particolarmente 
sofisticati sulle equazioni rappresentative.
*)

Options[graficoAnimazioneFascio] = {DisplayFunction -> $DisplayFunction};

graficoAnimazioneFascio[retta_Equal | retta_List, intervallox_List, intervalloy_List, 
   intervallok_List, opts___Rule] := 
   Module[{opzioni}, opzioni = DisplayFunction /.{opts} /.Options[graficoAnimazioneFascio]; 
      Table[Show[ImplicitPlot[retta, intervallox, intervalloy, FilterOptions[ImplicitPlot, opts], 
                 DisplayFunction -> Identity], 
      Graphics[Text["param = " <> ToString[Chop[First[intervallok]]], 
         {1/4(3 intervallox[[3]] + intervallox[[2]]), 1/4(3 intervalloy[[3]] + intervalloy[[2]])}], 
         DisplayFunction -> Identity], PlotRange -> {intervallox[[{2, 3}]], intervalloy[[{2, 3}]]}, 
         FilterOptions[Graphics, opts], DisplayFunction -> opzioni], intervallok]
   ]
      
graficoAnimazioneFascio[retta_Equal | retta_List, intervallox_List, intervalloy_List, 
   intervallok_List, {x0_, y0_}, opts___Rule] := 
   Module[{opzioni}, opzioni = DisplayFunction /.{opts} /.Options[graficoAnimazioneFascio]; 
      Table[Show[ImplicitPlot[retta, intervallox, intervalloy, FilterOptions[ImplicitPlot, opts], 
                 DisplayFunction -> Identity], 
        Graphics[{RGBColor[1, 0, 0], PointSize[0.02], Point[{x0, y0}]}], 
        Graphics[Text["param = " <> ToString[Chop[First[intervallok]]], 
           {1/4(3 intervallox[[3]] + intervallox[[2]]), 1/4(3 intervalloy[[3]] + intervalloy[[2]])}], 
           DisplayFunction -> Identity], PlotRange -> {intervallox[[{2, 3}]], intervalloy[[{2, 3}]]}, 
           FilterOptions[Graphics, opts], DisplayFunction -> opzioni], intervallok]
   ]
        
Options[graficoRetteParametriche] = Options[graficoRette] = {DisplayFunction -> $DisplayFunction};
    
graficoRetteParametriche[rette_List, param_Symbol, intervallox_List, intervalloy_List, opts___Rule] := 
   Module[{opzioni},  opzioni = DisplayFunction /. {opts} /. Options[graficoRetteParametriche]; 
      ImplicitPlot[(Eliminate[#1, param] &) /@ Cases[rette, _List], intervallox, intervalloy,
         FilterOptions[ImplicitPlot, opts], DisplayFunction -> opzioni]
   ]
      
graficoRette[rette_List, param_Symbol, intervallox_List, intervalloy_List, opts___Rule] := 
   Module[{opzioni}, opzioni = DisplayFunction /. {opts} /. Options[graficoRette]; 
      ImplicitPlot[(Eliminate[#1, param] &) /@ rette, intervallox, intervalloy, 
         FilterOptions[ImplicitPlot, opts], DisplayFunction -> opzioni]
   ]
      
graficoRette[rette_List, intervallox_List, intervalloy_List, opts___Rule] := 
   Module[{opzioni}, opzioni = DisplayFunction /. {opts} /. Options[graficoRette]; 
      ImplicitPlot[Cases[rette, _Equal], intervallox, intervalloy, 
         FilterOptions[ImplicitPlot, opts], DisplayFunction -> opzioni]
   ]
      
centroFascio[retta_, param_Symbol, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{primoMembro = First[retta] - Last[retta], sol}, 
      sol = Solve[Map[# == 0&, CoefficientList[primoMembro, param]], {x, y}];
      If[sol != {}, {x, y} /. Flatten[ sol, 1], sol]
   ] 
       
centroFascio[retta_ /;testLinearita[retta], param_Symbol] := 
   Module[{primoMembro = First[retta] - Last[retta], sol}, 
      sol = Solve[Map[# == 0&, CoefficientList[primoMembro, param]], {Global`x, Global`y}]; 
      If[sol != {}, {Global`x, Global`y} /. Flatten[ sol, 1], sol]
   ]
      
generatriciFascio[fascio_Equal, param_Symbol] := 
   Module[{primoMembro = First[fascio] - Last[fascio]}, 
      Map[# == 0&, CoefficientList[primoMembro, param]]
   ];
      
graficoFascioGeneratrici[fascio_Equal, intervallox_List, intervalloy_List, intervallok_List] := 
   Module[{listGeneratrici}, 
      listGeneratrici = graficoRette[generatriciFascio[fascio, First[intervallok]], intervallox, intervalloy, 
                        PlotStyle -> {RGBColor[1, 0, 0], RGBColor[0, 0, 1]}, DisplayFunction -> Identity]; 
      (Show[{#1, listGeneratrici}, DisplayFunction -> $DisplayFunction]&) /@ 
      graficoAnimazioneFascio[fascio, intervallox, intervalloy, intervallok, 
                              centroFascio[fascio, First[intervallok], {intervallox[[1]], intervalloy[[1]]}], 
                              DisplayFunction -> Identity]
   ];

distanza2Punti[{x1_, y1_}, {x2_, y2_}] := Sqrt[(x2 - x1)^2 + (y2 - y1)^2]
          
puntoMedio[{x1_, y1_}, {x2_, y2_}] := {(x1 + x2)/2, (y1 + y2)/2}

puntiCollineari[{x1_, y1_}, {x2_, y2_}, {x3_, y3_}] := 
   Module[{a, b}, Det[List[{x1, y1}, {x2, y2}, {x3, y3}] /. {a_, b_} -> {a, b, 1}] == 0];

retta2PuntiParametrica[{x1_, y1_}, {x2_, y2_}, param_Symbol, {x_Symbol, y_Symbol}] := 
   {x == x1 + param*(x2 - x1), y == y1 + param*(y2 - y1)};
      
retta2PuntiParametrica[{x1_,y1_},{x2_,y2_}, param_Symbol] := 
   {Global`x == x1 + param*(x2-x1), Global`y == y1 + param*(y2-y1)};      
      
retta2Punti[{x1_, y1_}, {x2_, y2_}, {x_Symbol, y_Symbol}] := 
   Module[{a, b}, Collect[Det[List[{x, y}, {x1, y1}, {x2, y2}] /. {a_, b_} -> {a, b, 1}], {x, y}] == 0];
            
retta2Punti[{x1_, y1_}, {x2_, y2_}] := 
   Module[{a, b}, 
      Collect[Det[List[{Global`x, Global`y}, {x1, y1}, {x2, y2}] /. {a_, b_} -> {a, b, 1}], 
      {Global`x, Global`y}] == 0
   ];
                  
rettaPerpendicolare[{x0_, y0_}, retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta, {x, y}]]}, 
      a = Coefficient[primoMembro, x]; 
      b = Coefficient[primoMembro, y]; 
      equazioneImplicita[b(x - x0) - a(y - y0) == 0, {x, y}] 
   ];
   
rettaPerpendicolare[{x0_, y0_}, coeffAng_/;(coeffAng != 0), {x_Symbol, y_Symbol}] := 
   Module[{},                         
      equazioneImplicita[y - y0 == -1/coeffAng (x - x0)] 
   ];                    
      
rettaPerpendicolare[{x0_, y0_}, retta_ /; testLinearita[retta]] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta]]}, 
      a = Coefficient[primoMembro, Global`x]; 
      b = Coefficient[primoMembro, Global`y]; 
      equazioneImplicita[b(Global`x - x0) - a(Global`y - y0) == 0]
   ];
   
rettaPerpendicolare[{x0_, y0_}, coeffAng_/;(coeffAng != 0)] := 
   Module[{}, 
      equazioneImplicita[Global`y - y0 == -1/coeffAng (Global`x - x0)]
   ];   
      
rettaParallela[{x0_, y0_}, retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta, {x, y}]]}, 
      a = Coefficient[primoMembro, x];
      b = Coefficient[primoMembro, y]; 
      equazioneImplicita[a(x - x0) + b(y - y0) == 0, {x, y}] 
   ];
   
rettaParallela[{x0_, y0_}, coeffAng_, {x_Symbol, y_Symbol}] := 
   Module[{},                         
      equazioneImplicita[y - y0 == coeffAng (x - x0)] 
   ];      
      
rettaParallela[{x0_, y0_}, retta_ /; testLinearita[retta]] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta]]}, 
      a = Coefficient[primoMembro, Global`x]; 
      b = Coefficient[primoMembro, Global`y]; 
      equazioneImplicita[a(Global`x - x0) + b(Global`y - y0) == 0]
   ];
   
rettaParallela[{x0_, y0_}, coeffAng_] := 
   Module[{},                         
      equazioneImplicita[Global`y - y0 == coeffAng (Global`x - x0)] 
   ];   
      
distanzaPuntoRetta[{x0_, y0_}, retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta, {x, y}]]}, 
      a = Coefficient[primoMembro, x]; 
      b = Coefficient[primoMembro, y]; 
      Abs[primoMembro /. {x -> x0, y -> y0}]/Sqrt[a^2 + b^2]
   ];
      
distanzaPuntoRetta[{x0_, y0_}, retta_ /; testLinearita[retta]] := 
   Module[{a, b, primoMembro = First[equazioneImplicita[retta]]}, 
      a = Coefficient[primoMembro, Global`x]; 
      b = Coefficient[primoMembro, Global`y]; 
      Abs[primoMembro /. {Global`x -> x0, Global`y -> y0}]/Sqrt[a^2 + b^2]
   ];
      
puntoIntersezione[retta1_, retta2_, {x_Symbol, y_Symbol}] /; (testLinearita[retta1, {x, y}] && testLinearita[retta2, {x, y}]) := 
   Module[{eq1 = equazioneImplicita[retta1, {x, y}], 
      eq2 = equazioneImplicita[retta2, {x, y}], sol}, 
      sol = Flatten[Solve[{eq1, eq2}, {x, y}], 1]; 
      If[sol != {}, {x, y} /. sol, sol]
   ];
           
puntoIntersezione[{retta1_, retta2_}, {x_Symbol, y_Symbol}] /; (testLinearita[retta1, {x, y}] && testLinearita[retta2, {x, y}]) := 
   Module[{eq1 = equazioneImplicita[retta1, {x, y}], eq2 = equazioneImplicita[retta2, {x, y}], sol}, 
      sol = Flatten[Solve[{eq1, eq2}, {x, y}], 1]; 
      If[sol != {}, {x, y} /. sol, sol]
   ];
           
puntoIntersezione[retta1_ /; testLinearita[retta1], retta2_ /; testLinearita[retta2]] := 
   Module[{eq1 = equazioneImplicita[retta1], eq2 = equazioneImplicita[retta2], sol}, 
      sol = Flatten[Solve[{eq1, eq2}, {Global`x, Global`y}], 1]; 
      If[sol != {}, {Global`x, Global`y} /. sol, sol]
   ];
      
puntoIntersezione[{retta1_ /; testLinearita[retta1], retta2_ /; testLinearita[retta2]}] := 
   Module[{eq1 = equazioneImplicita[retta1], eq2 = equazioneImplicita[retta2], sol}, 
      sol = Flatten[Solve[{eq1, eq2}, {Global`x, Global`y}], 1]; 
      If[sol != {}, {Global`x, Global`y} /. sol, {}]
   ];
      
appartenenzaPuntoRetta[{x0_, y0_}, retta_, {x_Symbol, y_Symbol}]/;testLinearita[retta, {x, y}] := 
   (retta /. {x -> x0, y -> y0});
      
appartenenzaPuntoRetta[{x0_, y0_}, retta_ /; testLinearita[retta]] := 
   retta /. {Global`x -> x0, Global`y -> y0};
    
areaTriangolo[{x1_, y1_}, {x2_, y2_}, {x3_, y3_}] := 
   1/2 Abs[Det[List[{x1, y1}, {x2, y2}, {x3, y3}] /. {a_, b_} -> {a, b, 1}]];
    
areaTriangolo[{{x1_, y1_}, {x2_, y2_}, {x3_, y3_}}] := 
   1/2 Abs[Det[List[{x1, y1}, {x2, y2}, {x3, y3}] /. {a_, b_} -> {a, b, 1}]];

listaColori = { 
   {"azzurro" , RGBColor[0.94174,1.,1.] },
   {"blu" , RGBColor[0.,0.,1.] },
   {"marrone" , RGBColor[0.5,0.164693,0.164693] },
   {"cyan" , RGBColor[0.,1.,1.] },
   {"verdeSmeraldo" , RGBColor[0.,0.790002,0.340007] },
   {"oro" , RGBColor[1.,0.843104,0.] },
   {"verde" , RGBColor[0.,1.,0.] },
   {"magenta" , RGBColor[1.,0.,1.] },
   {"arancione" , RGBColor[1.,0.5,0.] },
   {"rosa" , RGBColor[1.,0.752907,0.796106] },
   {"rosso" , RGBColor[1.,0.,0.] },
   {"violetto" , RGBColor[0.559999,0.370006,0.599994] },
   {"giallo" , RGBColor[1.,1.,0.] }
}

colore[colore_] := Extract[listaColori, Flatten[Position[listaColori, ToString[colore]]] + {0, 1}];

coloriDisponibili := Sort[First[Transpose[listaColori]]]
    
End[ ];

EndPackage[ ];
